# -*- coding: utf-8 -*-
import logging

from bson.objectid import ObjectId, InvalidId
import mongokit

from pyramid.decorator import reify
from pyramid.httpexceptions import (
    HTTPBadRequest,
    HTTPCreated,
    HTTPNotFound,
    HTTPOk,
    )

from pyramid_mongokit import register_document, IMongoConnection


from pyramid_rest.resource import ResourceAdded


log = logging.getLogger(__name__)

__all__ = ['register_document', 'CollectionView', ]


def includeme(config):
    config.include('pyramid_mongokit')
    config.add_subscriber(resource_added, ResourceAdded)


def resource_added(event):
    resource = event.resource
    cls = getattr(resource, 'view_class', None)
    if cls is None or (not issubclass(cls, CollectionView)):
        return
    collection = getattr(cls, 'model_class', None)
    if collection is None:
        return

    mongo_conn = event.config.registry.getUtility(IMongoConnection)
    mongo_conn.register(collection)
    log.info('Registered collection %s on mongokit connection.', collection)


class CollectionView(object):

    model_class = None

    def __init__(self, context, request):
        self.context = context
        self.request = request
        if self.model_class is None:
            raise Exception('model_class cannot be None')

    def _get_ids_dict(self, identifiers):
        """Convert ``str`` identifiers to ``ObjectId``"""
        ids = identifiers.copy()
        if 'id' in ids:
            ids['_id'] = ids.pop('id')
        try:
            for key, value in ids.iteritems():
                try:
                    cls = self.model_class.structure[key]
                except KeyError:
                    cls = ObjectId
                ids[key] = cls(value)
        except (InvalidId, ValueError):
            log.debug('Invalid id in %r', identifiers, exc_info=True)
            raise HTTPBadRequest('Invalid id in: %r' % identifiers)

        return ids

    def _get_document_or_404(self, identifiers):
        document = self.document_cls.find_one(self._get_ids_dict(identifiers))
        if document is None:
            raise HTTPNotFound()
        return document

    @reify
    def collection(self):
        # model_class.__collection__ or model_class.__name__ as collection name
        collection_name = getattr(self.model_class, '__collection__')
        return getattr(self.request.mongo_db, collection_name)

    @reify
    def document_cls(self):
        return getattr(self.request.mongo_db, self.model_class.__name__)

    def index(self, **identifiers):
        parent_ids = self._get_ids_dict(identifiers)
        return {'data': list(self.collection.find(parent_ids))}

    def create(self, **identifiers):
        document = self.document_cls()
        document.update(self.request.POST)
        for k, v in self._get_ids_dict(identifiers).iteritems():
            document[k] = v
        try:
            document.save()
        except (mongokit.StructureError, mongokit.RequireFieldError):
            log.exception(
                'error while creating %s, POST: %s, ids: %s',
                self.context.resource,
                self.request.POST,
                identifiers,
                )
            raise HTTPBadRequest()

        ids = [identifiers[name]
               for name in self.context.resource.ids[:-1]]
        ids.append(document['_id'])

        location = self.request.rest_resource_url(
            self.context.resource.name,
            *ids
            )
        return HTTPCreated(location=location)

    def update(self, **identifiers):
        # XXX: full replace + etag + last-modified
        document = self._get_document_or_404(identifiers)
        for field, value in self.request.POST.iteritems():
            if field in identifiers or field not in document:
                continue
            document[field] = value
        document.save()
        return HTTPOk()

    def show(self, **identifiers):
        return self._get_document_or_404(identifiers)

    def delete(self, **identifiers):
        """No delete cascade on sub resources."""
        self._get_document_or_404(identifiers).delete()
        return HTTPOk()



# -*- coding: utf-8 -*-
import functools
import inspect
import logging

from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPMethodNotAllowed
from pyramid.config.util import ActionInfo

import venusian

from zope.interface import implementer
from zope.interface import Interface

log = logging.getLogger(__name__)


class ViewMapper(object):

    def __init__(self, **kwargs):
        self.kwargs = kwargs


class FunctionViewMapper(ViewMapper):

    def __call__(self, view):

        def wrapper(context, request):

            return view(
                context,
                request,
                **request.matchdict
                )

        return wrapper


class ClassViewMapper(ViewMapper):

    def __call__(self, view):

        def wrapper(context, request):
            return view(
                view.im_class(context, request),
                **request.matchdict
                )

        return wrapper


class IResourceConfigurator(Interface):
    pass


class ResourceAdded(object):
    """Event notified after a resource is added and configured."""
    def __init__(self, config, resource):
        self.config = config
        self.resource = resource


def not_allowed_view(request):
    # FIXME using raise makes tests fail
    return HTTPMethodNotAllowed()


@implementer(IResourceConfigurator)
class ResourceConfigurator(object):
    """Internal configurator of all REST resources. It is responsible for:

    # Registering to pyramid routes and views associated to resources.
    # . . .

    """

    methods = dict(
        index='GET',
        create='POST',
        show='GET',
        update='PUT',
        delete='DELETE',
        new='GET',
        edit='GET',
        )

    singular_methods = dict(
        show='GET',
        update='PUT',
        edit='GET',
        )

    def __init__(self):
        self.resources = dict()
        self.parent_resources = dict()
        self.deferred = dict()
        self.methods_configs = dict()

    def add_resource(
        self,
        config,
        resource_name,
        plural_name=None,
        singular=False,
        documentation="",
        acl=None,
        ):
        res = resource_config(
            resource_name,
            plural_name,
            singular,
            documentation,
            acl,
            )

        class_name = ''.join(a.title() for a in res.collection_name.split("."))
        module_name = res.collection_name.replace('.', '_')
        dotted_module = '%s.views.%s' % (config.package_name, module_name)
        dotted_class = '%s:%sView' % (dotted_module, class_name)
        try:
            cls = config.maybe_dotted(dotted_class)
        except ImportError:
            log.error('No class %s found.', dotted_class)
            raise
        config.scan(dotted_module)

        res(cls)  # decorate class to register config
        res.update_views_settings(self.methods_configs.get(cls, {}))
        self._add(config, res)

    def add_method_config(self, cls, method, settings):
        if cls not in self.methods_configs:
            self.methods_configs[cls] = dict()
        self.methods_configs[cls][method] = settings

    def _add(self, config, resource):
        # XXX detect resource name duplicates
        try:
            parent, child = resource.collection_name.rsplit('.', 1)
            if parent not in self.resources:
                self.deferred[resource.name] = resource
                return
        except ValueError:
            # no separator in resource name: it is a root resource
            parent = None
            child = resource.collection_name

        if parent is None:
            self.parent_resources[resource.name] = resource
            resource.parent = None
            resource.depth = 0
        else:
            # resource knows about parent
            resource.parent = self.resources[parent]
            # parent knows about new child
            resource.parent.children[resource.name] = resource
            resource.depth = resource.parent.depth + 1

        if resource.parent:
            parent_pattern = resource.parent.lineage_route_pattern
        else:
            parent_pattern = ''

        if resource.singular:
            # route names
            resource.route_name = resource.name
            resource.edit_route_name = '%s_edit' % resource.name
            resource.item_route_name = None
            resource.new_route_name = None

            # routes patterns
            resource.pattern = '%s/%s' % (parent_pattern, child)
            resource.edit_pattern = '%s/edit' % resource.pattern
            resource.item_pattern = None
            resource.new_pattern = None

        else:
            # routes names:
            resource.route_name = "%s_collection" % resource.name
            resource.item_route_name = "%s_item" % resource.name
            resource.new_route_name = '%s_new' % resource.name
            resource.edit_route_name = '%s_edit' % resource.name

            # routes patterns
            resource.pattern = '%s/%s' % (parent_pattern, child)
            resource.item_pattern = '%s/{id}' % resource.pattern
            resource.new_pattern = '%s/new' % resource.pattern
            resource.edit_pattern = '%s/edit' % resource.item_pattern

        if config._ainfo is None:
            config._ainfo = []

        # XXX: to be refactored
        # self._validate_views_args_name(resource)
        self._add_routes(config, resource)
        self._add_views(config, resource)
        self._add_introspectable(config, resource)
        self.resources[resource.name] = resource
        log.info(
            'Add REST resource="%s" parent="%s" views=%s patterns=[%s, %s]',
            resource.name,
            resource.parent.name if resource.parent else None,
            resource.views.keys(),
            resource.pattern,
            resource.item_pattern,
            )
        config.registry.notify(ResourceAdded(config, resource))
        self._add_deferred_children(config, resource)

    def _add_deferred_children(self, config, parent_resource):
        for name, child_resource in self.deferred.items():
            parent_name, child_name = name.rsplit('.', 1)
            if parent_name == parent_resource.name:
                self.deferred.pop(name)
                self._add(config, child_resource)

    def _validate_views_args_name(self, resource):
        # XXX seems ugly: need refacto.
        not_last = resource.ids[:-1]

        expectation = {
            'index': not_last,
            'create': not_last,
            'show': resource.ids,
            'update': resource.ids,
            'delete': resource.ids,
            'new': not_last,
            'edit': resource.ids,
            }

        for view_info in resource.views.itervalues():
            spec = inspect.getargspec(view_info.view)
            if inspect.isfunction(view_info.view):
                expected = (
                    ['context', 'request']
                    + expectation[view_info.method]
                    )
            else:
                expected = ['self'] + expectation[view_info.method]
            nb_args = len(spec.args) - (
                len(spec.defaults) if spec.defaults else 0
                )
            received = spec.args[:nb_args]
            if expected != received:
                raise TypeError(
                    'resource=%s, view=%s, expected %s - got %s' % (
                    resource,
                    view_info.method,
                    expected,
                    received,
                    ))

    def _add_routes(self, config, resource):
        config._ainfo.append(ActionInfo(*resource.info.codeinfo))
        factory = functools.partial(ResourceContext, resource)
        if not resource.singular:
            config.add_route(
                pattern='%s' % resource.new_pattern,
                name='%s' % resource.new_route_name,
                factory=factory,
                )
            config.add_route(
                pattern=resource.item_pattern,
                name=resource.item_route_name,
                factory=factory,
                )
        config.add_route(
            pattern='%s' % resource.edit_pattern,
            name='%s' % resource.edit_route_name,
            factory=factory,
            )
        config.add_route(
            pattern=resource.pattern,
            name=resource.route_name,
            factory=factory,
            )
        config._ainfo.pop()

    def _add_views(self, config, resource):
        config._ainfo.append(ActionInfo(*resource.info.codeinfo))
        for view_info in resource.views.itervalues():
            if inspect.isfunction(view_info.view):
                mapper = FunctionViewMapper
                attr = None
            else:
                mapper = ClassViewMapper
                attr = view_info.view.__name__
            settings = view_info.settings.copy()
            settings.update(
                self._get_view_predicates(resource, view_info.method)
                )
            settings.setdefault('renderer', 'pyramid_rest_renderer')
            config.add_view(
                view=view_info.view,
                mapper=mapper,
                attr=attr,
                **settings
                )

        if not resource.singular:
            not_allowed = [m for m in self.methods if m not in resource.views]
        if resource.singular:
            not_allowed = [m for m in self.singular_methods if (
                m not in resource.views)
                ]

        for method in not_allowed:
            config.add_view(
                view=not_allowed_view,
                **self._get_view_predicates(resource, method)
                )
        config._ainfo.pop()

    def _add_introspectable(self, config, resource):
        cat_name = 'pyramid_rest resources'
        intr = config.introspectable(
            category_name=cat_name,
            discriminator=resource.discriminator,
            title=resource.name,
            type_name='resource',
            )
        intr['resource'] = resource
        intr['documentation'] = resource.__doc__
        intr.relate('routes', resource.route_name)
        intr.relate('routes', resource.edit_route_name)
        if not resource.singular:
            intr.relate('routes', resource.item_route_name)
            intr.relate('routes', resource.new_route_name)

        if resource.parent:
            intr.relate(cat_name, resource.parent.discriminator)

        config._ainfo.append(ActionInfo(*resource.info.codeinfo))
        config.action(resource.discriminator, introspectables=(intr,))
        config._ainfo.pop()

    def _get_view_predicates(self, resource, method):
        if not resource.singular:
            route_name = {
                'index': resource.route_name,
                'create': resource.route_name,
                'show': resource.item_route_name,
                'delete': resource.item_route_name,
                'update': resource.item_route_name,
                'edit': resource.edit_route_name,
                'new': resource.new_route_name,
                }[method]
        if resource.singular:
            route_name = {
                'show': resource.route_name,
                'update': resource.route_name,
                'edit': resource.edit_route_name,
                }[method]
        return dict(
            route_name=route_name,
            permission=method,
            request_method=self.methods[method],
            )

    def resource_path(self, request, resource_name, *args):
        """Returns route name for ``resource_name`` with args"""
        resource = self.resources[resource_name]
        route_name = resource.get_route_name(*args)
        ids = dict(zip(
            resource.ids[:len(args)],
            args,
            ))
        return request.route_path(route_name, **ids)


def rest_resource_url(request, resource_name, *args):
    utility = request.registry.getUtility(IResourceConfigurator)
    path = utility.resource_path(request, resource_name, *args)
    return request.host_url + path


def rest_resource_path(request, resource_name, *args):
    utility = request.registry.getUtility(IResourceConfigurator)
    return utility.resource_path(request, resource_name, *args)


class ViewInfo(object):

    def __init__(self, view, info, method, settings):
        self.view = view
        self.info = info
        self.method = method
        self.settings = settings


class BaseResource(object):
    """
        :param resource_name: Resource name in singular form.
        :param plural_name: Resource collection name: a *s* is appended to
                                *resource_name* if it's not provided.
    """

    methods = tuple(ResourceConfigurator.methods.keys())
    singular_methods = tuple(ResourceConfigurator.singular_methods.keys())

    def __init__(
        self,
        resource_name,
        plural_name=None,
        singular=False,
        documentation="",
        acl=None,
        ):

        self.parent = None
        self.acl = acl
        self.name = resource_name
        self.singular = singular
        self.short_name = self.name.rpartition('.')[-1]

        if not self.singular and plural_name:
            name = list(resource_name.rpartition('.'))
            name[-1] = plural_name
            self.collection_name = ''.join(name)

        if not self.singular and not plural_name:
            self.collection_name = '%ss' % resource_name

        if self.singular:
            self.collection_name = self.name

        self.views = dict()
        self._conflicts = dict()
        self.children = dict()
        self.__doc__ = documentation

    def __repr__(self):
        return "<%s '%s'>" % (self.__class__.__name__, self.name)

    @reify
    def ids(self):
        """List of identifier names for this resource.

        Example::

            >>> resource = Resource('application.user')
            >>> resource.ids
            ['application_id', 'id']
        """
        my_id = [] if self.singular else ['id']
        if self.parent:
            return self.lineage_ids[:-1] + my_id
        return my_id

    @reify
    def lineage_ids(self):
        """List of identifier names a child resource needs for its route
        pattern. e.g.::

            >>> resource = Resource('application.user.message')
            >>> resource.parent.lineage_ids
            ['application_id', 'user_id']
        """
        id_name = self.short_name + '_id'
        if self.parent:
            return self.parent.lineage_ids + [id_name]
        return [id_name]

    @reify
    def lineage_route_pattern(self):
        child = self.collection_name.rpartition('.')[2]
        pattern = '/%s/{%s_id}' % (child, self.short_name)
        if self.parent:
            return self.parent.lineage_route_pattern + pattern
        return pattern

    @property
    def discriminator(self):
        return ('pyramid_rest', repr(self))

    def callback(self, context, name, ob):
        config = context.config.with_package(self.info.module)
        config.registry.getUtility(IResourceConfigurator)._add(config, self)

    def get_route_name(self, *args):
        """Returns route name associated with that resource and the arguments
        for url generation

        :param args: List of ids.
        """
        if (len(args) < self.depth
            or len(args) > self.depth + 1
            or (self.singular and len(args) != self.depth)
            ):
            raise ValueError('Need between %s and %s ids, received %s' % (
                self.depth,
                self.depth + 1,
                len(args),
                ))
        if self.singular:
            return self.route_name

        is_collection_path = (len(args) == self.depth)
        is_item_path = (len(args) == self.depth + 1)

        if is_collection_path:
            return self.route_name

        if is_item_path:
            return self.item_route_name


class Resource(BaseResource):
    """Resource class"""

    def __init__(
        self,
        resource_name,
        plural_name=None,
        singular=False,
        documentation="",
        acl=None,
        ):
        super(Resource, self).__init__(
            resource_name,
            plural_name,
            singular,
            documentation,
            acl,
            )
        self.info = venusian.attach(self, self.callback)

        # define REST decorators
        if not self.singular:
            iter_methods = iter(self.methods)

        if self.singular:
            iter_methods = iter(self.singular_methods)

        for method in iter_methods:
            setattr(self, method, functools.partial(self.decorator, method))

    def decorator(self, method, **kwargs):
        def wrapper(view):
            info = venusian.attach(view, self.callback_view)
            view_info = ViewInfo(view, info, method, kwargs)
            self.views[method] = view_info
            self._conflicts[view] = view_info
            return view
        return wrapper

    def callback_view(self, context, name, view):
        view_info = self._conflicts[view]
        config = context.config
        config._ainfo.append(ActionInfo(*view_info.info.codeinfo))
        config.action((self.name, view_info.method))
        config._ainfo.pop()
        self._conflicts.pop(view)


class resource_config(BaseResource):
    """Resource decorator"""

    def __call__(self, cls):
        self.info = venusian.attach(cls, self.callback, category='pyramid')
        self.view_class = cls
        self.__doc__ = cls.__doc__
        methods = self.singular_methods if self.singular else self.methods
        views = inspect.getmembers(
            cls,
            lambda m: inspect.ismethod(m) and m.__name__ in methods
            )
        for name, view in views:
            self.views[name] = ViewInfo(view, self.info, name, {})
        return cls

    def update_views_settings(self, method_configs):
        for method_name, settings in method_configs.iteritems():
            self.views[method_name].settings.update(settings)

    def callback(self, context, name, ob):
        config = context.config.with_package(self.info.module)
        utility = config.registry.getUtility(IResourceConfigurator)
        method_configs = utility.methods_configs.get(ob, {})
        self.update_views_settings(method_configs)
        super(resource_config, self).callback(context, name, ob)


class method_config(object):

    def __init__(self, **settings):
        self.settings = settings

    def __call__(self, method):
        self.info = venusian.attach(method, self.callback)
        self.method = method
        return method

    def callback(self, context, name, ob):
        # callback may be called twice as 2 scans might be done:
        ## 1st is done when in ResourceConfigurator.add_resource
        ## 2nd might be done by the end user
        if self.method is not None:
            config = context.config.with_package(self.info.module)
            utility = config.registry.getUtility(IResourceConfigurator)
            utility.add_method_config(ob, self.method.func_name, self.settings)
            self.method = None


class ResourceContext(object):

    def __init__(self, resource, request):
        self.__parent__ = resource.parent
        self.__name__ = resource.name
        self.__acl__ = resource.acl if resource.acl else tuple()
        self.resource = resource
        self.request = request



# -*- coding: utf-8 -*-
from datetime import datetime, date
from decimal import Decimal
import json
import logging

from bson import BSON
from bson.objectid import ObjectId


log = logging.getLogger(__name__)


def json_dumps(o):
    return json.dumps(o, cls=JSONEncoder)


def json_loads(s):
    return json.loads(s, parse_float=Decimal)


def bson_dumps(o):
    return BSON.encode(o)


def bson_loads(s):
    return BSON(s).decode()


class Factory(object):

    #XXX Permit to extend formats?

    formatters = {
        'application/json': json_dumps,
        'application/bson': bson_dumps,
        }

    def __init__(self, info):
        """ Constructor: info will be an oect having the
        following attributes: name (the renderer name), package
        (the package that was 'current' at the time the
        renderer was registered), type (the renderer type
        name), registry (the current application registry) and
        settings (the deployment settings dictionary). """
        self.info = info

    def __call__(self, value, system):
        """ Call the renderer implementation with the value
        and the system value passed in as arguments and return
        the result (a string or unicode oect).  The value is
        the return value of a view.  The system value is a
        dictionary containing available system values
        # (e.g. view, context, and request). """
        request = system['request']
        format = request.accept.best_match([
            'application/json',
            'application/bson',
            ])
        request.response.content_type = format
        return self.formatters[format](value)


class JSONEncoder(json.JSONEncoder):
    """Custom encoder that supports extra types.

    Supported types: date, datetime, Decimal, bson.objectid.ObjectId.
    """

    def default(self, o):
        if isinstance(o, (datetime, date)):
            return o.isoformat()

        if isinstance(o, Decimal):
            return _number_str(o)

        if isinstance(o, ObjectId):
            return str(o)

        return super(JSONEncoder, self).default(o)


class _number_str(float):
    # kludge to have decimals correctly output as JSON numbers;
    # converting them to strings would cause extra quotes

    def __init__(self, o):
        self.o = o

    def __repr__(self):
        return str(self.o)



from example.views import BaseView


class ApplicationUserScoreView(BaseView):

    def update(self, application_id, user_id):
        pass

    def show(self, application_id, user_id):
        return {'user_id': user_id, 'score': 123}

    def edit(self, application_id, user_id):
        pass



# -*- coding: utf-8 -*-
import unittest

import mock


class TestFunctionViewMapper(unittest.TestCase):

    def test_init_and_call(self):
        from pyramid_rest.resource import FunctionViewMapper

        kwargs = dict()
        view = mock.Mock()
        ctx = mock.Mock()
        req = mock.MagicMock()
        req.matchdict.__iter__.return_value = ('id1', 'id0', 'id3', 'id2')

        fvm = FunctionViewMapper(**kwargs)
        wrapped = fvm(view)
        result = wrapped(ctx, req)

        self.assertEqual(kwargs, fvm.kwargs)
        self.assertEqual(view.return_value, result)

        view.assert_called_once_with(ctx, req, **req.matchdict)


class TestResourceConfigurator(unittest.TestCase):

    def _get_config(self):
        from pyramid.config import Introspectable, Configurator, Registry
        config = mock.Mock(spec=Configurator)
        config.introspectable.return_value = mock.MagicMock(
            spec=Introspectable
            )
        config.registry = mock.MagicMock(spec=Registry)
        return config

    def test_init(self):
        from pyramid_rest.resource import ResourceConfigurator

        ru = ResourceConfigurator()

        self.assertEqual(
            dict(
                index='GET',
                create='POST',
                show='GET',
                update='PUT',
                delete='DELETE',
                new='GET',
                edit='GET',
                ),
            ru.methods,
            )

    def test_add_sub_resource(self):
        from pyramid_rest.resource import ResourceConfigurator, Resource

        dad = Resource('dad')
        kid = Resource('dad.kid')

        config = self._get_config()

        # kid is a sub resource:
        # Resource utility must defer processing until parent resource is added
        ru = ResourceConfigurator()
        ru._add(config, kid)

        self.assertEqual({'dad.kid': kid}, ru.deferred)

        ru._add(config, dad)

    @mock.patch('pyramid_rest.resource.functools')
    def test_add_resources_route(self, m_functools):
        from pyramid_rest.resource import ResourceConfigurator, Resource

        kid = Resource('dad.kid')
        dad = Resource('dad')

        config = self._get_config()

        ru = ResourceConfigurator()

        ru._add(config, dad)

        self.assertEqual({'dad': dad}, ru.resources)
        self.assertEqual({'dad': dad}, ru.parent_resources)

        self.assertEqual(
            mock.call(
                pattern='/dads/new',
                name='dad_new',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[0]
            )
        self.assertEqual(
            mock.call(
                pattern='/dads/{id}',
                name='dad_item',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[1]
            )
        self.assertEqual(
            mock.call(
                pattern='/dads/{id}/edit',
                name='dad_edit',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[2]
            )
        self.assertEqual(
            mock.call(
                pattern='/dads',
                name='dad_collection',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[3]
            )
        ru._add(config, kid)
        self.assertEqual({'dad': dad, 'dad.kid': kid}, ru.resources)
        self.assertEqual({'dad': dad}, ru.parent_resources)

        self.assertEqual(
            mock.call(
                pattern='/dads/{dad_id}/kids/new',
                name='dad.kid_new',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[4]
            )
        self.assertEqual(
            mock.call(
                pattern='/dads/{dad_id}/kids/{id}',
                name='dad.kid_item',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[5]
            )
        self.assertEqual(
            mock.call(
                pattern='/dads/{dad_id}/kids/{id}/edit',
                name='dad.kid_edit',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[6]
            )
        self.assertEqual(
            mock.call(
                pattern='/dads/{dad_id}/kids',
                name='dad.kid_collection',
                factory=m_functools.partial.return_value,
                ),
            config.add_route.call_args_list[7]
            )

    def test_add_resources_add_views(self):
        from pyramid_rest.resource import (
            ResourceConfigurator,
            Resource,
            not_allowed_view,
            )

        dad_index = mock.Mock(spec=not_allowed_view)
        dad_show = mock.Mock(spec=not_allowed_view)

        kid_index = mock.Mock(spec=not_allowed_view)
        kid_show = mock.Mock(spec=not_allowed_view)

        kid = Resource('dad.kid')
        dad = Resource('dad')

        config = self._get_config()

        ru = ResourceConfigurator()
        ru._validate_views_args_name = lambda a: a

        # simulate decorating methods:
        dad.index()(dad_index)
        dad.show()(dad_show)
        kid.index()(kid_index)
        kid.show()(kid_show)

        ru._add(config, dad)
        ru._add(config, kid)

        self.assertEqual(14, config.add_view.call_count)

        # check dad views:

        self._check_add_view(dad_index, 'index', 'GET', 'dad_collection', config.add_view.call_args_list[0])
        self._check_add_view(dad_show, 'show', 'GET', 'dad_item', config.add_view.call_args_list[1])

        self._check_add_not_allowed(not_allowed_view, 'edit', 'GET', 'dad_edit', config.add_view.call_args_list[2])
        self._check_add_not_allowed(not_allowed_view, 'new', 'GET', 'dad_new', config.add_view.call_args_list[3])
        self._check_add_not_allowed(not_allowed_view, 'create', 'POST', 'dad_collection', config.add_view.call_args_list[4])
        self._check_add_not_allowed(not_allowed_view, 'update', 'PUT', 'dad_item', config.add_view.call_args_list[5])
        self._check_add_not_allowed(not_allowed_view, 'delete', 'DELETE', 'dad_item', config.add_view.call_args_list[6])

        # check kid views:

        self._check_add_view(kid_index, 'index', 'GET', 'dad.kid_collection', config.add_view.call_args_list[7])
        self._check_add_view(kid_show, 'show', 'GET', 'dad.kid_item', config.add_view.call_args_list[8])

        self._check_add_not_allowed(not_allowed_view, 'edit', 'GET', 'dad.kid_edit', config.add_view.call_args_list[9])
        self._check_add_not_allowed(not_allowed_view, 'new', 'GET', 'dad.kid_new', config.add_view.call_args_list[10])
        self._check_add_not_allowed(not_allowed_view, 'create', 'POST', 'dad.kid_collection', config.add_view.call_args_list[11])
        self._check_add_not_allowed(not_allowed_view, 'update', 'PUT', 'dad.kid_item', config.add_view.call_args_list[12])
        self._check_add_not_allowed(not_allowed_view, 'delete', 'DELETE', 'dad.kid_item', config.add_view.call_args_list[13])

    def _check_add_view(self, view, permission, request_method, route_name, real_call):
        from pyramid_rest.resource import FunctionViewMapper
        self.assertEqual(
            mock.call(
                view=view,
                mapper=FunctionViewMapper,
                attr=None,
                renderer='pyramid_rest_renderer',
                permission=permission,
                request_method=request_method,
                route_name=route_name
                ),
            real_call,
            )

    def _check_add_not_allowed(self, view, permission, request_method, route_name, real_call):
        self.assertEqual(
            mock.call(
                view=view,
                permission=permission,
                request_method=request_method,
                route_name=route_name
                ),
            real_call,
            )


class TestNotAllowedView(unittest.TestCase):

    def test_raising_http_not_allowed(self):
        from pyramid_rest.resource import not_allowed_view, HTTPMethodNotAllowed
        self.assertIsInstance(not_allowed_view(None), HTTPMethodNotAllowed, )


class TestResource(unittest.TestCase):

    @mock.patch('pyramid_rest.resource.functools')
    def test_init(self, m_functools):
        from pyramid_rest.resource import Resource

        r = Resource('dad')

        self.assertEqual(m_functools.partial.return_value, r.index)
        self.assertEqual(m_functools.partial.return_value, r.show)
        self.assertEqual(m_functools.partial.return_value, r.create)
        self.assertEqual(m_functools.partial.return_value, r.update)
        self.assertEqual(m_functools.partial.return_value, r.delete)
        self.assertEqual(m_functools.partial.return_value, r.new)
        self.assertEqual(m_functools.partial.return_value, r.edit)

        self.assertEqual('<Resource \'dad\'>', r.__repr__())

    @mock.patch('pyramid_rest.resource.ViewInfo')
    @mock.patch('pyramid_rest.resource.venusian')
    def test_decorator(self, m_venusian, m_ViewInfo):
        from pyramid_rest.resource import Resource

        r = Resource('dad')

        view_index = mock.Mock()

        wrapper = r.decorator('index')
        result = wrapper(view_index)

        self.assertEqual(view_index, result)
        self.assertEqual(
            {'index': m_ViewInfo.return_value},
            r.views
            )

        self.assertEqual(
            mock.call(r, r.callback),
            m_venusian.attach.call_args_list[0]
            )
        self.assertEqual(
            mock.call(view_index, mock.ANY),
            m_venusian.attach.call_args_list[1]
            )

    def test_callback(self):
        from pyramid_rest.resource import Resource, IResourceConfigurator

        context = mock.Mock()

        r = Resource('dad')

        r.callback(context, None, None)

        context.config.with_package.assert_called_once_with(
            r.info.module
            )
        config = context.config.with_package.return_value

        (config
            .registry
            .getUtility
            .assert_called_once_with(IResourceConfigurator)
            )

        (config
            .registry
            .getUtility
            .return_value
            ._add
            .assert_called_once_with(config, r)
            )

    @mock.patch('pyramid_rest.resource.ActionInfo')
    def test_callback_view(self, m_action_info):
        from pyramid_rest.resource import Resource

        context = mock.Mock()
        view = mock.Mock()

        r = Resource('dad')
        r.decorator('index')(view)

        view_info = r._conflicts[view]

        r.callback_view(context, None, view)

        context.config._ainfo.append.assert_called_once_with(
            m_action_info.return_value,
            )
        context.config.action.assert_called_once_with(('dad', 'index'))
        context.config._ainfo.pop.assert_called_once_with()

        m_action_info.assert_called_once_with(*view_info.info.codeinfo)

    def test_lineage_ids(self):
        from pyramid.config import Configurator
        from pyramid_rest.resource import Resource, ResourceConfigurator

        config = Configurator(settings={})
        ru = ResourceConfigurator()

        dad = Resource('dad')
        kid = Resource('dad.kid')
        grandchild = Resource('dad.kid.grandchild')

        ru._add(config, dad)
        ru._add(config, kid)
        ru._add(config, grandchild)

        self.assertEqual(['dad_id'], dad.lineage_ids)
        self.assertEqual(['dad_id', 'kid_id'], kid.lineage_ids)
        self.assertEqual(['dad_id', 'kid_id', 'grandchild_id'], grandchild.lineage_ids)

        self.assertEqual(['id'], dad.ids)
        self.assertEqual(['dad_id', 'id'], kid.ids)
        self.assertEqual(['dad_id', 'kid_id', 'id'], grandchild.ids)


class TestResourceContext(unittest.TestCase):

    def test_init(self):
        from pyramid_rest.resource import ResourceContext, Resource

        resource = Resource('dad')
        request = mock.Mock()

        rctx = ResourceContext(resource, request)

        self.assertEqual(resource, rctx.resource)
        self.assertEqual(request, rctx.request)



# -*- coding: utf-8 -*-
import unittest
from decimal import Decimal
from bson.objectid import ObjectId


class TestJsonDump(unittest.TestCase):

    def test_dumps(self):
        from pyramid_rest.renderer import json_dumps
        result = json_dumps({'key': Decimal('0.2')})

        self.assertEqual(
            '{"key": 0.2}',
            result
            )


class TestBson(unittest.TestCase):

    def test_encode_decode(self):
        from pyramid_rest.renderer import bson_dumps, bson_loads

        o = {
            '_id': ObjectId('50aab36b9978d001f3be41b6'),
            'unicode': u"yéééé",
            'float': 1.2,
            }

        result = bson_loads(bson_dumps(o))

        self.assertEqual(o, result)


class TestRestFactory(unittest.TestCase):

    def test_call(self):
        from pyramid_rest.renderer import Factory



import unittest

import mock


def wrong_ids_show(context, request, app_id, u_id, id):
    "Should have been application_id, user_id, id"
    return {}


class TestRoutePatternIds(unittest.TestCase):

    def _get_config(self):
        from pyramid.config import Configurator, Registry
        config = mock.MagicMock(spec=Configurator)
        config.registry = mock.MagicMock(spec=Registry)
        return config

    @unittest.expectedFailure
    def test_wrong_parent_id_names(self):
        from pyramid_rest.resource import Resource, ResourceConfigurator

        config = self._get_config()
        ru = ResourceConfigurator()

        # add parents:
        ru._add(config, Resource('application'))
        ru._add(config, Resource('application.user'))

        res = Resource('application.user.message')
        # decorate view function
        res.show()(wrong_ids_show)

        with self.assertRaises(TypeError):
            # simulate venusian callback
            ru._add(config, res)



from pyramid_rest.resource import Resource, resource_config, method_config

_USERS = {}

users = Resource('user')


@users.index()
def list_users(context, request):
    return _USERS


@users.create()
def create_user(context, request):
    pass


@users.show()
def get_user(context, request, id):
    return {'id': 123}

messages = Resource('user.message')


@messages.index()
def get_user_messages(context, request, user_id):
    return


user_summary = Resource('user.summary', singular=True)


@user_summary.show()
def get_user_summary(context, request, user_id):
    return {'summary': {}}


@resource_config('user.score', singular=True)
class UserScore(object):

    def __init__(self, context, request):
        pass

    def show(self, user_id):
        return {}

    @method_config(renderer='example.mako')
    def edit(self, user_id):
        return {}



# -*- coding: utf-8 -*-
import datetime

from bson.objectid import ObjectId

from mongokit import Document


class Message(Document):

    use_dot_notation = True

    __collection__ = 'example'

    structure = {
        'from': unicode,
        'content': unicode,
        'application_id': ObjectId,
        'user_id': ObjectId,
        'created': datetime.datetime,
        }

    default_values = {
        'created': datetime.datetime.utcnow,
        }

    required_fields = ['from', 'content', 'application_id', 'user_id']

    indexes = [{'fields': ['application_id', 'user_id']}]



# -*- coding: utf-8 -*-
import unittest


class TestMongo(unittest.TestCase):

    def test_no_document(self):
        from pyramid_rest.mongo import CollectionView
        with self.assertRaises(Exception):
            CollectionView(None, None)



# -*- coding: utf-8 -*-
import os
import sys
import unittest

import webtest

from pyramid import testing


class TestExampleController(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()
        self.config.include('example')
        self.config.commit()
        self.app = webtest.TestApp(self.config.make_wsgi_app())

    @property
    def mongo_db(self):
        from pyramid_rest.mongo import IMongoConnection
        return getattr(
            self.app.app.registry.getUtility(IMongoConnection),
            os.environ['MONGO_DB_NAME']
            )

    def tearDown(self):
        testing.tearDown()
        for m in list(sys.modules):
            if 'example' in m:
                del sys.modules[m]



# -*- coding: utf-8 -*-
import unittest

import mock


class TestIncludeme(unittest.TestCase):

    @mock.patch('pyramid_rest.ResourceConfigurator')
    def test_includeme(self, m_utility):
        from pyramid_rest import (
            includeme,
            RequestMethodEventPredicate,
            override_request_method,
            NewRequest,
            )

        config = mock.Mock()
        config.registry.settings.get.return_value = 'true'

        includeme(config)

        config.registry.registerUtility.assert_called_once_with(
            m_utility.return_value
            )

        self.assertEqual(
            mock.call('pyramid_rest.tunneling', 'true', ),
            config.registry.settings.get.call_args_list[0],
            )

        config.add_subscriber_predicate.assert_called_once_with(
            'request_methods',
            RequestMethodEventPredicate
            )

        config.add_subscriber.assert_called_once_with(
            override_request_method,
            NewRequest,
            request_methods=['POST'],
            )

    def test_includeme_no_tunnelling(self):
        from pyramid_rest import includeme

        config = mock.Mock()
        config.registry.settings.get.return_value = 'false'

        includeme(config)

        self.assertFalse(config.add_subscriber_predicate.called)
        self.assertFalse(config.add_subscriber.called)


class TestRequestMethodEventPredicate(unittest.TestCase):

    def test_predicate(self):
        from pyramid_rest import RequestMethodEventPredicate

        pred = RequestMethodEventPredicate(['PUT', 'DELETE'], None)

        self.assertEqual(
            'request_method in %s' % ['PUT', 'DELETE'],
            pred.text()
            )

        self.assertEqual(['PUT', 'DELETE'], pred.methods)

        event = mock.Mock()
        event.request.method = 'PUT'

        self.assertTrue(pred(event))

        event.request.method = 'DELETE'
        self.assertTrue(pred(event))

        event.request.method = 'POST'
        self.assertFalse(pred(event))


class TestOverridRequestMethod(unittest.TestCase):

    def test_override_with_header(self):
        from pyramid_rest import override_request_method

        event = mock.Mock()
        event.request.headers.get.return_value = 'PUT'

        override_request_method(event)

        self.assertEqual('PUT', event.request.method)

    def test_override_with_query_param(self):
        from pyramid_rest import override_request_method

        event = mock.Mock()
        event.request.headers.get.return_value = None
        event.request.GET.get.return_value = 'DELETE'

        override_request_method(event)

        self.assertEqual('DELETE', event.request.method)

    def test_wrong_override(self):
        from pyramid_rest import override_request_method

        event = mock.Mock()
        event.request.headers.get.return_value = 'POST'
        event.request.method = 'GET'

        override_request_method(event)

        self.assertEqual('GET', event.request.method)



# -*- coding: utf-8 -*-

from pyramid_rest.mongo import CollectionView

from example.model import Application


class ApplicationsView(CollectionView):

    model_class = Application

    def index(self):
        return {
            'data': [
                dict(id=1, name='App 1'),
                dict(id=2, name='App 2'),
                dict(id=3, name='App 3'),
                dict(id=4, name='App 4'),
                ],
            }

    def update(self, id):
        return {}



# -*- coding: utf-8 -*-
from pyramid_rest.tests.functional import TestExampleController


class TestRenderer(TestExampleController):

    def test_bson_renderer(self):
        from pyramid_rest.renderer import json_loads, bson_loads

        result1 = self.app.get(
            '/applications',
            headers={'Accept': 'application/bson'},
            )
        result1 = bson_loads(result1.body)

        result2 = self.app.get(
            '/applications',
            headers={'Accept': 'application/json'},
            )
        result2 = json_loads(result2.body)

        self.assertEqual(result1, result2)

        expected = {
            u'data': [
                {u'id': 1, u'name': u'App 1'},
                {u'id': 2, u'name': u'App 2'},
                {u'id': 3, u'name': u'App 3'},
                {u'id': 4, u'name': u'App 4'},
                ]
            }

        self.assertEqual(expected, result1)



# -*- coding: utf-8 -*-
from sphinx.util.compat import Directive

from pyramid.config import Configurator

def setup(app):
    app.add_directive('rest_resource', ResourceDirective)


def convert_to_list(argument):
    """Convert a comma separated list into a list of python values"""
    if argument is None:
        return []
    else:
        return [i.strip() for i in argument.split(',')]

def convert_to_list_required(argument):
    if argument is None:
        raise ValueError('argument required but none supplied')
    return convert_to_list(argument)





class ResourceDirective(Directive):

    has_content = True
    option_spec = {
        'modules': convert_to_list_required,
        }
    domain = 'pyramid_rest'


    def __init__(self, *args, **kwargs):
        super(ResourceDirective, self).__init__(*args, **kwargs)
        self.env = self.state.document.settings.env
        self.config = Configurator(settings=None)
        self.config.include('pyramid_rest')

    def run(self):
        self.scan()
        return []

    def scan(self):
        for module in self.options.get('modules'):
            self.config.scan(module)
        self.config.commit()



# -*- coding: utf-8 -*-


from example.model.application import Application
from example.model.user import User
from example.model.message import Message

__all__ = [
    'Application',
    'User',
    'Message',
    ]



# -*- coding: utf-8 -*-

from pyramid_rest.resource import method_config

from pyramid_rest.mongo import CollectionView

from example.model import User


class ApplicationUsersView(CollectionView):

    model_class = User

    def index(self, application_id):
        return [
            dict(id='1', name='User 1'),
            dict(id='2', name='User 2'),
            dict(id='3', name='User 3'),
            dict(id='4', name='User 4'),
            ]

    @method_config(permission='admin')
    def show(self, application_id, id):
        return dict(id=id, name='User %s' % id)



# -*- coding: utf-8 -*-

from mongokit import Document


class Application(Document):

    use_dot_notation = True

    __collection__ = 'example'

    structure = {
        'name': unicode,
        'secret_key': unicode,
        }

    required_fields = ['name', 'secret_key']



# -*- coding: utf-8 -*-
import functools
import logging

from pyramid.events import NewRequest
from pyramid.settings import asbool

from pyramid_rest.resource import (
    ResourceConfigurator,
    rest_resource_url,
    rest_resource_path,
    )

log = logging.getLogger(__name__)


class RequestMethodEventPredicate(object):

    def __init__(self, methods, config):
        self.methods = methods
        self.phash = self.text

    def __call__(self, event):
        return event.request.method in self.methods

    def text(self):
        return 'request_method in %s' % self.methods


def override_request_method(event):
    methods = ['PUT', 'DELETE']
    override = (
        event.request.headers.get('X-HTTP-Method-Override') or
        event.request.GET.get('_method', '').upper()
        )

    if override in methods:
        event.request.method = override


def includeme(config):
    log.info('Including pyramid_rest')

    utility = ResourceConfigurator()

    config.registry.registerUtility(utility)

    config.add_renderer(
        'pyramid_rest_renderer',
        'pyramid_rest.renderer.Factory'
        )

    config.add_directive('add_resource', utility.add_resource)
    config.add_directive(
        'add_singular_resource',
        functools.partial(utility.add_resource, singular=True),
        )
    config.add_request_method(rest_resource_url, 'rest_resource_url')
    config.add_request_method(rest_resource_path, 'rest_resource_path')

    if asbool(config.registry.settings.get('pyramid_rest.tunneling', 'true')):
        config.add_subscriber_predicate(
            'request_methods',
            RequestMethodEventPredicate
            )

        config.add_subscriber(
            override_request_method,
            NewRequest,
            request_methods=['POST'],
            )

    if asbool(config.registry.settings.get('pyramid_rest.mongo', 'true')):
        config.include('pyramid_rest.mongo')

    config.commit()



# -*- coding: utf-8 -*-
import unittest

from pyramid import testing

from pyramid_rest.tests.functional import TestExampleController


class TestResource(TestExampleController):

    def test_method_not_allowed(self):
        self.app.put('/users/123', status=405)

    @unittest.expectedFailure
    def test_application(self):
        self.app.get('/applications/1', status=405)
        self.app.get('/applications/1/edit', status=405)
        self.app.get('/applications/new', status=405)
        self.app.post('/applications', status=405)

        self.assertEqual({
            'data': [
                dict(id=1, name='App 1'),
                dict(id=2, name='App 2'),
                dict(id=3, name='App 3'),
                dict(id=4, name='App 4'),
                ],
            },
            self.app.get('/applications').json,
            )

        self.assertEqual({}, self.app.put('/applications/1').json)
        self.assertEqual({}, self.app.post('/applications/1?_method=PUT').json)

        self.assertEqual({}, self.app.delete('/applications/1').json)
        self.assertEqual(
            {},
            self.app.post('/applications/1?_method=DELETE').json
            )

    @unittest.expectedFailure
    def test_application_users(self):
        self.assertEqual([
            dict(id='1', name='User 1'),
            dict(id='2', name='User 2'),
            dict(id='3', name='User 3'),
            dict(id='4', name='User 4'),
            ],
            self.app.get('/applications/1/users').json
            )

        self.assertEqual(
            dict(id='1', name='User 1'),
            self.app.get('/applications/1/users/1').json,
            )

        self.app.post('/applications/1/users', status=405)
        self.app.put('/applications/1/users/1', status=405)
        self.app.get('/applications/1/users/1/edit', status=405)
        self.app.get('/applications/1/users/new', status=405)
        self.app.post('/applications/1/users', status=405)

    def test_application_media(self):
        self.app.get('/applications/1/media', status=405)
        self.app.get('/applications/1/media/1', status=405)
        self.app.get('/applications/1/media/new', status=405)
        self.app.get('/applications/1/media/edit', status=405)

        self.app.post('/applications/1/media', status=405)
        self.app.put('/applications/1/media/1', status=405)
        self.app.delete('/applications/1/media/1', status=405)

        self.app.post('/applications/1/media/1?_method=DELETE', status=405)

    def test_view_not_found(self):
        with self.assertRaises(ImportError):
            self.config.add_resource('no.such.resource')

    def test_resource_get_collection_paths(self):
        from pyramid_rest.resource import IResourceConfigurator
        utility = self.config.registry.getUtility(IResourceConfigurator)
        application_users = utility.resources['application.user']

        self.assertRaises(ValueError, application_users.get_route_name)

        self.assertEqual(
            'application.user_collection',
            application_users.get_route_name(1)
            )

        self.assertEqual(
            'application.user_item',
            application_users.get_route_name(1, 2)
            )

    def test_resource_get_singular_path(self):
        from pyramid_rest.resource import IResourceConfigurator
        utility = self.config.registry.getUtility(IResourceConfigurator)
        score = utility.resources['application.user.score']

        self.assertRaises(ValueError, score.get_route_name)
        self.assertRaises(ValueError, score.get_route_name, 1,)
        self.assertRaises(ValueError, score.get_route_name, 1, 2, 3)

        self.assertEqual(
            'application.user.score',
            score.get_route_name(1, 2)
            )

    def test_resource_path(self):
        from pyramid_rest.resource import IResourceConfigurator
        request = testing.DummyRequest()
        utility = self.config.registry.getUtility(IResourceConfigurator)
        self.assertEqual(
            '/applications/1/users/2/score',
            utility.resource_path(request, 'application.user.score', 1, 2)
            )



"""
Just an example RESTfull app for documentating and testing purpose.
"""
from pyramid.config import Configurator
from wsgiref.simple_server import make_server


def includeme(config):
    config.include('pyramid_rest')
    config.include('pyramid_debugtoolbar')

    config.add_resource('application')
    config.add_resource('application.medium', plural_name='media')
    config.add_resource('application.user')
    config.add_resource('application.user.message')

    config.add_singular_resource('application.user.score')
    config.add_singular_resource('health')

    config.scan()


def get_app(settings=None):
    settings if settings else {}
    config = Configurator(settings=settings)
    config.include('example')
    return config.make_wsgi_app()


def main(global_config, **settings):
    return get_app(settings)


if __name__ == '__main__':
    server = make_server('0.0.0.0', 6543, get_app())
    server.serve_forever()



#!/usr/bin/env python
import setuptools

if not getattr(setuptools, "_distribute", False):
    raise SystemExit("""Setuptools is not supported. Please use Distribute""")

setup_requires = [
    'd2to1',
    ]

tests_require = [
    'coverage',
    'mock',
    'mongokit',
    'nose',
    'nosexcover',
    'webtest',
    'waitress',
    'yanc',
    ]

extras_require = {'mongo': ['pyramid_mongokit']}

setuptools.setup(
    setup_requires=setup_requires,
    tests_require=tests_require,
    extras_require=extras_require,
    d2to1=True,
    package_data={
        },
    entry_points="""
        [paste.app_factory]
            main = pyramid_rest:main
        """,
    paster_plugins=['pyramid'],
    )



# -*- coding: utf-8 -*-

from bson.objectid import ObjectId

from mongokit import Document


class User(Document):

    use_dot_notation = True

    __collection__ = 'example'

    structure = {
        'name': unicode,
        'application_id': ObjectId,
        }

    required_fields = ['application_id', 'name']

    indexes = [{'fields': ['application_id']},
               {'fields': ['name'], 'unique': True},
               ]



# -*- coding: utf-8 -*-
import datetime

from bson.objectid import ObjectId
from pyramid.httpexceptions import HTTPBadRequest

from pyramid_rest.tests.functional import TestExampleController


class TestCollectionView(TestExampleController):

    def setUp(self):
        super(TestCollectionView, self).setUp()

        app = self.mongo_db.Application()
        app.name = u'An Application?'
        app.secret_key = u'secret'

        app.save()

        user = self.mongo_db.User()
        user.name = u'Bob Marley'
        user.application_id = app._id
        user.save()

        self.app_id = str(app._id)
        self.user_id = str(user._id)

    def tearDown(self):
        self.mongo_db.application.drop()
        self.mongo_db.user.drop()
        self.mongo_db.message.drop()

        super(TestCollectionView, self).tearDown()

    def test_create(self):
        result = self.app.post(
            '/applications/%s/users/%s/messages' % (self.app_id, self.user_id),
            {u'from': u'Peter Tosh', u'content': u'yo man'}
            )

        json_body = self.app.get(result.location).json

        message = self.mongo_db.Message.find_one(
            {'_id': ObjectId(json_body['_id'])}
            )
        self.assertTrue(message)
        self.assertEqual(u'Peter Tosh', message['from'])
        self.assertEqual(u'yo man', message.content)
        self.assertEqual(self.app_id, str(message.application_id))
        self.assertEqual(self.user_id, str(message.user_id))

        expected = 'http://localhost/applications/%s/users/%s/messages/%s' % (
            self.app_id,
            self.user_id,
            message['_id'],
            )
        self.assertEqual(expected, result.location)

    def test_create_missing_required(self):
        with self.assertRaises(HTTPBadRequest):
            self.app.post(
                '/applications/%s/users/%s/messages' % (
                    self.app_id,
                    self.user_id,
                    ),
                {u'from': u'Peter Tosh'},
                status=400
                )

    def test_update(self):
        message = self.mongo_db.Message(doc={
            'application_id': ObjectId(self.app_id),
            'user_id': ObjectId(self.user_id),
            'from': u'Peter Tosh',
            'content': u'yo man',
            'created': datetime.datetime.utcnow(),
            })
        message.save()

        self.app.put(
            '/applications/%s/users/%s/messages/%s' % (
                self.app_id, self.user_id, str(message._id)
                ),
            {u'content': u'yo man 2'}
            )

        result = self.mongo_db.Message.find_one({'_id': message._id})
        self.assertEqual(u'yo man 2', result.content)

    def test_delete(self):
        pass

    def test_create_conflict(self):
        result = self.app.post(
            '/applications/%s/users' % self.app_id,
            {u'name': u'Bob Marley', }
            )
        print result
